var generators = require('yeoman-generator');
var path = require('path');
var _ = require('lodash');

module.exports = generators.Base.extend({

  prompting: function() {
    return this.prompt([{
      type: 'input',
      name: 'component',
      message: 'Your component name',
      default: this.appname // Default to current folder name
    }, {
      type: 'input',
      name: 'authorName',
      message: 'Your name'
    }]).then(function(answers) {
      this.lowerCaseName = answers.component.toLowerCase();
      this.componentName = _.upperFirst(answers.component);
      this.authorName = answers.authorName;
    }.bind(this));
  },

  // copy templates
  configuring: function() {
    this.fs.copyTpl(
      this.templatePath('_package.json'),
      this.destinationPath('package.json'),
      {
        lowerCaseName: this.lowerCaseName,
        componentName: this.componentName,
        authorName: this.authorName
      }
    );
    this.fs.copyTpl(
      this.templatePath('_index.html'),
      this.destinationPath('./index.html'),
      { componentName: this.componentName }
    );
    this.fs.copy(
      this.templatePath('_webpack.config.js'),
      this.destinationPath('./webpack.config.js')
    );
    this.fs.copy(
      this.templatePath('_babelrc'),
      this.destinationPath('./.babelrc')
    );
    this.fs.copy(
      this.templatePath('_eslintrc'),
      this.destinationPath('./.eslintrc')
    );
    this.fs.copy(
      this.templatePath('_gitignore'),
      this.destinationPath('./.gitignore')
    );
    this.fs.copy(
      this.templatePath('_main.js'),
      this.destinationPath('./main.js')
    );
    this.fs.copy(
      this.templatePath('_LICENSE'),
      this.destinationPath('./LICENSE')
    );
    this.fs.copy(
      this.templatePath('test/_mocha.opts'),
      this.destinationPath('./test/mocha.opts')
    );
    this.fs.copy(
      this.templatePath('test/_setup.js'),
      this.destinationPath('./test/setup.js')
    );
    this.fs.copyTpl(
      this.templatePath('_App.js'),
      this.destinationPath('./App.js'),
      { componentName: this.componentName }
    );
    this.fs.copyTpl(
      this.templatePath('_index.js'),
      this.destinationPath('./index.js'),
      { componentName: this.componentName }
    );
    this.fs.copyTpl(
      this.templatePath('_README.md'),
      this.destinationPath('./README.md'),
      { componentName: this.componentName }
    );
    this.fs.copyTpl(
      this.templatePath('src/_component.jsx'),
      this.destinationPath('./src/' + this.componentName + '.jsx'),
      { componentName: this.componentName }
    );
    this.fs.copyTpl(
      this.templatePath('src/_component.css'),
      this.destinationPath('./src/' + this.componentName + '.css'),
      { componentName: this.componentName }
    );
    this.fs.copyTpl(
      this.templatePath('test/_component.spec.js'),
      this.destinationPath('./test/' + this.componentName + '.spec.js'),
      { componentName: this.componentName }
    );
  },

  install: function() {
    this.npmInstall(
      [
        'react',
        'react-dom'
      ],
      { 'save': false }
    );
    this.npmInstall(
      [
        // packages for transpile ES6
        'babel-core',
        'babel-cli',
        'babel-loader',
        'babel-preset-es2015',
        'babel-preset-react',
        'babel-preset-react-hmre',
        'babel-preset-stage-0',
        'babel-plugin-module-alias',
        'ignore-styles',
        // packages for pack js files
        'webpack',
        'webpack-dev-server',
        'style-loader',
        'css-loader',
        'url-loader',
        'file-loader',
        'postcss',
        'postcss-loader',
        'postcss-cssnext',
        'postcss-cli',
        'precss',
        // packages for package.json scripts
        'ghooks',
        'npm-run-all',
        // packages for test environment
        'mocha',
        'expect',
        'expect-jsx',
        'react-addons-test-utils',
        'enzyme',
        'jsdom',
        // lint
        'eslint',
        'babel-eslint',
        'eslint-config-airbnb',
        'eslint-plugin-import',
        'eslint-plugin-jsx-a11y',
        'eslint-plugin-react'
      ],
      { 'saveDev': true }
    );
  }
});
